use strict;
use ExtUtils::MakeMaker;
#use Config qw(%Config);
use lib ".";

$| = 1;

my $DEBUG = 0;
my %opts;
my $i = 0;
foreach (@ARGV) {
	my ($k, $v) = split /=/, $_, 2;
	$opts{$k} = $v;
	unless (exists $ExtUtils::MakeMaker::Recognized_Att_Keys{$k}) {
		splice(@ARGV, $i, 1);
	} else {
		$i++;
	}
}


my $scmxx = `which scmxx 2>/dev/null`;
chomp $scmxx;
$scmxx ||= "";
my @devices = &get_devices();
my @baudrate = qw(9600 19200 38400 57600 115200);

my %prefs = (
	SCMXX => $scmxx,
	DEVICE => $devices[0] || '/dev/ttyS0',
	BAUDRATE => $baudrate[1],
	TIMEOUT => 20,
	PBTIMEOUT => 30,
	ABTIMEOUT => 60
);


if ($opts{INTERACTIVE}) {
	my $x = 0;
	print "\n";
	my $choice = $scmxx;
	while ($x == 0) {
		$choice = prompt("Enter path to scmxx: ", $choice);
		if (!-f $choice) {
			print "$choice does not exist or is not a regular file!\n";
			$choice = $scmxx;
			sleep(1);
		}  elsif (!-x $choice) {
			$choice = $scmxx;
			print "$choice is not executable!\n";
			sleep(1);
		} else {
			$x = 1;
		}
	}
	$prefs{SCMXX} = $choice;
	print "\n";

	$x = 0;
	$choice = 1;
	while ($x == 0) {
		my $list = join ("\n   ", map { ($_+1) . ") $devices[$_] " } (0..$#devices) );
		$choice = prompt ("Which device do you want to use?\n\n   " 
						. $list 
						. "\n\nEnter the appropriate number or device: ", $choice);

		if ($choice !~ /^\d+/) {
			my $f = 0;
			foreach my $i (0..$#devices) {
				if ($choice eq $devices[$i]) {
					$choice = $i;
					$f = 1;
					$x = 1;
					last;
				}
			}
			unless ($f) {
				print "Please enter a number (1-" . ($#devices+1) . ")\n";
				$choice = 1;
				sleep(1);
			}
		} elsif ($choice-1 > $#devices) {
			print "$choice is out of range (1-" . ($#devices+1) . ")\n";
			$choice = 1;
			sleep(1);
		} else {
			$x = 1;
		}
	}
	$prefs{DEVICE} = $devices[$choice-1];
	print "\n";

	$x = 0;
	$choice = 2;
	while ($x == 0) {
		my $list = join ("\n   ", map { ($_+1) . ") $baudrate[$_] " } (0..$#baudrate) );
		$choice = prompt ("Which Baudrate do you want to use?" 
						. "\n   (This MUST equal the following setting in the ME45/S45: "
						. "\n   Menu -> Setup -> Connectivity -> Baud Rate)\n\n   "
						. $list 
						. "\nEnter the appropriate number: ", $choice);

		if ($choice !~ /^\d+/) {
			print "Please enter a number (1-" . ($#baudrate+1) . ")\n";
			$choice = 2;
			sleep(1);
		} elsif ($choice-1 > $#baudrate) {
			my $f = 0;
			foreach my $i (0..$#baudrate) {
				if ($choice eq $baudrate[$i]) {
					$choice = $i;
					$f = 1;
					$x = 1;
					last;
				}
			}
			unless ($f) {
				print "$choice is out of range (1-" . ($#baudrate+1) . ")\n";
				$choice = 2;
				sleep(1);
			}
		} else {
			$x = 1;
		}
	}
	$prefs{BAUDRATE} = $baudrate[$choice-1];
	print "\n";
} else {
	unless ($prefs{SCMXX}) {
		print <<EOT;

Warning: 'scmxx' was not found in your path.
If you have installed 'scmxx' do a 

    perl Makefile.PL INTERACTIVE=1

to set the path of 'scmxx'. Otherwise get it from
http://www.hendrik-sattler.de/scmxx and install it.

EOT
#		exit(1); # "Can't build SCMxx Module");
		sleep(3);
	}
}

if ($prefs{$_} =~ /ircomm/) {
	$prefs{TIMEOUT} = 40;
	$prefs{PBTIMEOUT} = 60;
	$prefs{ABTIMEOUT} = 120;
}
open (CFG, ">SCMxx/Config.pm") || die "Can't open SCMxx/Config.pm $!";
print CFG qq|\$SCMxx::Config = {\n|;
print CFG qq|\t"$_"\t=> "$prefs{$_}",\n| foreach keys (%prefs);
#print CFG qq|\t"TIMEOUT"\t=>20\n|;
print CFG qq|};\n1;\n__END__\n|;
close(CFG);
 
#open (CFG, ">SCMxx.pm.cfg") || die "Can't open SCMxx.pm.cfg";
#print CFG qq|\$config = {\n|;
#print CFG qq|"$_" => "$prefs{$_}",\n| foreach keys (%prefs);
#print CFG qq|};\n|;
#close(CFG);
 
# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.
WriteMakefile(
	'NAME'			=> 'SCMxx',
	'VERSION_FROM'	=> 'SCMxx.pm', # finds $VERSION
	'PREREQ_PM'		=> {}, # e.g., Module::Name => 1.1
	'dist'			=> { COMPRESS=>'gzip',SUFFIX=>'gz' },
);


# try to detect usable devices 
sub get_devices {

	# check for usable ttyS* devices 
	# see http://www.ibiblio.org/mdw/HOWTO/Serial-HOWTO-10.html#ss10.1
	# this will only work with linux!
	print "Checking for tty-serial ports .......... " if $DEBUG;
	my @ttyS;
	my $procfile = "/proc/tty/driver/serial";
	if (-e $procfile) {
		open (PROC, "$procfile") || die "Can't open $procfile $!";
		while (<PROC>) {
			chomp;
			if ($_ !~ /unknown port/ && $_ !~ /serinfo/ ) {
				my ($num, undef) = split (/:/, $_, 2);
				push @ttyS, "/dev/ttyS$num";
			}
		}
		close (PROC);
		if (@ttyS) {
			print "available\n" if $DEBUG;
			print "	found: " . join(", ", @ttyS) if $DEBUG;
			print "\n" if $DEBUG;
		}
	} else {
		print "$procfile not available\n" if $DEBUG;
	}

	#TODO
	# check for useable ircomm devices
	# for now just list all of them 
	my @ircoms = `ls -1 /dev/ircomm*`;
	chomp $_ foreach (@ircoms);

	# check if usb is present
	my @usbtty;
	print "Checking for usb support ............... " if $DEBUG;
	if (-e "/proc/bus/usb/drivers") {
		print "OK\n" if $DEBUG;
		# check if usbserial is present
		print "Checking for usbserial support ......... " if $DEBUG;
		my $usbser = `grep serial /proc/bus/usb/drivers`;
		if ($usbser =~ /serial/) {
			print "available\n" if $DEBUG;
			print "Checking for usbserial device .......... " if $DEBUG;
			$usbser = `grep -c serial /proc/bus/usb/devices`;
			chomp $usbser;
			$usbser =~ s/\s+//g;
			if ($usbser > 0) {
				print "available\n" if $DEBUG;
				print "	 $usbser device(s) found\n" if $DEBUG;
				print "	 assuming devices " if $DEBUG;
				my $devpath;
				if (-d "/dev/usb/tts") {
					$devpath = "/dev/usb/tts";
				} else {
					$devpath = "/dev/usb/ttyUSB";
				}
				foreach (1..$usbser) {
					push @usbtty, "$devpath" . ($_-1);
				}
				print join (", ", @usbtty) . "\n" if $DEBUG;
			} else {
				print "NOT available\n" if $DEBUG;
			}
		} else {
			print "NOT available\n" if $DEBUG;
		}
	} else {
		print "NOT available\n" if $DEBUG;
	}

	print "\n\n" if $DEBUG; 
	return (@ttyS, @usbtty, @ircoms);
}

=comment
package MY;

sub processPL {
    my($self) = shift;
    my @m;
    return "" unless $self->{'PL_FILES'};
    my %files = %{$self->{'PL_FILES'}};
	foreach my $from (sort keys %{$self->{PL_FILES}}) {
    	my $to = $self->{PL_FILES}->{$from};
		my $cfg = $self->{PL_FILES}->{$from} . ".cfg";
		push @m, <<EOT; 
pm_to_blib: $to

$to :: $from
	\$(PERL) -I\$(INST_ARCHLIB) -I\$(INST_LIB) \\
	-I\$(PERL_ARCHLIB) -I\$(PERL_LIB) -Ilib -I../lib\\
	-MExtUtils::PerlPP -e 'ppp("$from", "$to", "$cfg")'
EOT
	}
	join "", @m;
}
=comment
    push(@output, <<"PART");
pm_to_blib: $to

$to :: $from
	\$(PERL) -I\$(INST_ARCHLIB) -I\$(INST_LIB) -I\$(PERL_ARCHLIB) \\
		-I\$(PERL_LIB) -Ilib -I../lib -MExtUtils::PerlPP \\
		-e ppp "$from" "$to" "$cfg"

PART
    }
    join "", @output;
}
=cut

1;

